import 'bootstrap.css';
import '@grapecity/wijmo.styles/wijmo.css';
import './styles.css';
//
import * as wjCore from '@grapecity/wijmo';
import * as wjGrid from '@grapecity/wijmo.grid';
import '@angular/compiler';
import { Component, Inject, enableProdMode, NgModule, ViewChild, AfterViewInit } from '@angular/core';

import { platformBrowserDynamic } from '@angular/platform-browser-dynamic';
import { BrowserModule } from '@angular/platform-browser';
import { WjGridModule } from '@grapecity/wijmo.angular2.grid';
import { DataService } from './app.data';

//
@Component({
    selector: 'app-component',
    templateUrl: 'src/app.component.html'
})
export class AppComponent implements AfterViewInit {
    @ViewChild('theGrid', { static: true }) theGrid: wjGrid.FlexGrid;
    data: any;
    rowCountText: string;
    // DataSvc will be passed by derived classes
    constructor(@Inject(DataService) private dataService: DataService) {
        this.data = dataService.getData();
    }

    initGrid(grid: wjGrid.FlexGrid){
        grid.formatItem.addHandler((s,e) => {
            if(e.panel == s.cells){
                let item = e.getRow().dataItem;
                if(item && item['$_dummyNode']){
                    // shows loader in cell
                    e.cell.innerHTML = '<div class="lds-dual-ring"></div>';
                    // 
                    wjCore.addClass(e.cell, 'dummy-node')
                }
            }
        })
    }

    //
    ngAfterViewInit() {
        this.theGrid.collapseGroupsToLevel(0);
        window['gr'] = this.theGrid;
        this._updateRowCount(this.theGrid);
    }
    //
    async onGroupCollapsedChanged(s: wjGrid.FlexGrid, e: wjGrid.CellRangeEventArgs) {
        var row = s.rows[e.row] as wjGrid.GroupRow,
            item = row.dataItem;
        //
        // did we just expand a node with a dummy child?
        if (!row.isCollapsed &&
            item.children.length == 1 &&
            item.children[0].name == '') {
            //
            // can't lazy load while updating rows
            if (s.rows.isUpdating) {
                row.isCollapsed = true;
                return;
            }
            //
            // replace the dummy child with actual nodes
            item.children.length = 0;

            // simulate API call to get node data 
            new Promise((resolve, error) => {
                setTimeout(() => {
                    resolve(this.dataService.getNodes(item.id))
                }, 3000)
            }).then((nodes) => {
                console.log('node nodes is: ', nodes);
                (nodes as any).forEach((node: any) => {
                    // insert dummy node for each received node
                    node.children.push(this.dataService.createNode(true))
                    // 
                    item.children.push(node);
                });
                // refresh the view
                s.collectionView.refresh();

                // collapse the new item's child items
                this.collapseItems(s, row);

                // update row count
                this._updateRowCount(s);
            })
        }
    }

    collapseItems(s: wjGrid.FlexGrid, row: wjGrid.GroupRow) {
        for (var i = <number>row.index + 1; i < s.rows.length; i++) {
            var childRow = s.rows[i] as wjGrid.GroupRow;
            if (childRow.level <= row.level) {
                break;
            }
            childRow.isCollapsed = true;
        }
    }

    private _updateRowCount(grid: wjGrid.FlexGrid) {
        this.rowCountText = wjCore.Globalize.format(grid.rows.length, 'n0');
    }
}
//\\
@NgModule({
    imports: [WjGridModule, BrowserModule],
    declarations: [AppComponent],
    providers: [DataService],
    bootstrap: [AppComponent]
})
export class AppModule {
}
//
enableProdMode();
// Bootstrap application with hash style navigation and global services.
platformBrowserDynamic().bootstrapModule(AppModule);
